# SPDX-FileCopyrightText: Copyright (c) 2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: MIT

from dataclasses import dataclass, field
from typing import Any, List, Optional, Union

from air_sdk.v2.air_model import AirModel, ApiNotImplementedMixin, BaseEndpointApi, PrimaryKey
from air_sdk.v2.endpoints import mixins
from air_sdk.v2.endpoints.organizations import Organization
from air_sdk.v2.utils import validate_payload_types


@dataclass(eq=False)
class FleetWorker(ApiNotImplementedMixin, AirModel):
    """Worker data returned in fleet responses"""

    id: str
    url: str
    cpu: int
    memory: int
    storage: int
    ip_address: str
    port_range: str
    region: str
    available: bool
    fqdn: str
    fleet: str
    contact: Union[List[str], str]
    capabilities: str
    cpu_arch: str

    @classmethod
    def get_model_api(cls):
        return FleetWorkerEndpointApi


class FleetWorkerEndpointApi(BaseEndpointApi[FleetWorker]):
    API_PATH = 'workers'  # placeholder
    model = FleetWorker

    def get(self, pk: PrimaryKey, **params: Any) -> FleetWorker:
        return self.load_model({'id': str(pk)})


@dataclass(eq=False)
class Fleet(AirModel):
    id: str
    name: str
    organization: Optional[Organization] = field(metadata=AirModel.FIELD_FOREIGN_KEY, repr=False)
    prefix: str = field(repr=False)
    prefix_length: int = field(repr=False)
    gateway_ipv4: Optional[str] = field(repr=False)
    port_range: str = field(repr=False)
    container_ipv4_network: str = field(repr=False)
    container_prefix: int = field(repr=False)
    labels: List[str] = field(repr=False)
    public: bool = field(repr=False)
    workers: Optional[List[FleetWorker]] = field(repr=False)

    @classmethod
    def get_model_api(cls):
        """
        Returns the respective `AirModelAPI` type for this model.
        """
        return FleetEndpointApi

    @validate_payload_types
    def update(
        self,
        name: Optional[str] = None,
        organization: Optional[Union[Organization, PrimaryKey]] = None,
        prefix: Optional[str] = None,
        prefix_length: Optional[int] = None,
        gateway_ipv4: Optional[str] = None,
        port_range: Optional[str] = None,
        container_ipv4_network: Optional[str] = None,
        container_prefix: Optional[int] = None,
        labels: Optional[List[str]] = None,
    ) -> None:
        data = {
            'name': name,
            'organization': organization,
            'prefix': prefix,
            'prefix_length': prefix_length,
            'gateway_ipv4': gateway_ipv4,
            'port_range': port_range,
            'container_ipv4_network': container_ipv4_network,
            'container_prefix': container_prefix,
            'labels': labels,
        }
        data = {key: value for (key, value) in data.items() if value is not None}
        super().update(**data)

    @validate_payload_types
    def full_update(
        self,
        name: str,
        organization: Union[Organization, PrimaryKey],
        prefix: Optional[str] = None,
        prefix_length: Optional[int] = None,
        gateway_ipv4: Optional[str] = None,
        port_range: Optional[str] = None,
        container_ipv4_network: Optional[str] = None,
        container_prefix: Optional[int] = None,
        labels: Optional[List[str]] = None,
    ) -> None:
        data = {
            'name': name,
            'organization': organization,
            'prefix': prefix,
            'prefix_length': prefix_length,
            'gateway_ipv4': gateway_ipv4,
            'port_range': port_range,
            'container_ipv4_network': container_ipv4_network,
            'container_prefix': container_prefix,
            'labels': labels,
        }
        data = {key: value for (key, value) in data.items() if value is not None}
        super().update(**data)


class FleetEndpointApi(
    mixins.ListApiMixin[Fleet],
    mixins.CreateApiMixin[Fleet],
    mixins.GetApiMixin[Fleet],
    mixins.PatchApiMixin[Fleet],
    mixins.PutApiMixin[Fleet],
    mixins.DeleteApiMixin,
    BaseEndpointApi[Fleet],
):
    API_PATH = 'fleet'
    model = Fleet

    @validate_payload_types
    def create(
        self,
        name: str,
        organization: Union[Organization, PrimaryKey],
        prefix: Optional[str] = None,
        prefix_length: Optional[int] = None,
        gateway_ipv4: Optional[str] = None,
        port_range: Optional[str] = None,
        container_ipv4_network: Optional[str] = None,
        container_prefix: Optional[int] = None,
        labels: Optional[List[str]] = None,
    ) -> Fleet:
        data = {
            'name': name,
            'organization': organization,
            'prefix': prefix,
            'prefix_length': prefix_length,
            'gateway_ipv4': gateway_ipv4,
            'port_range': port_range,
            'container_ipv4_network': container_ipv4_network,
            'container_prefix': container_prefix,
            'labels': labels,
        }
        data = {key: value for (key, value) in data.items() if value is not None}
        return super().create(**data)



# SPDX-FileCopyrightText: Copyright (c) 2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: MIT

from dataclasses import dataclass, field
from typing import List, Optional, Union

from air_sdk.v2.air_model import AirModel, BaseEndpointApi, DataDict, PrimaryKey
from air_sdk.v2.endpoints import mixins
from air_sdk.v2.endpoints.accounts import Account
from air_sdk.v2.endpoints.images import Image
from air_sdk.v2.endpoints.organizations import Organization


@dataclass(eq=False)
class Manifest(AirModel):
    id: str = field(repr=False)
    artifacts_directory: str = field(repr=False)
    artifacts_directory_max_size_gb: int = field(repr=False)
    boot_group: int = field(repr=False)
    configure_node_properties: DataDict = field(repr=False)
    configure_simulator: DataDict = field(repr=False)
    docker_run_parameters: DataDict = field(repr=False)
    emulation_type: str
    organization: Optional[Organization] = field(metadata=AirModel.FIELD_FOREIGN_KEY, repr=False)
    owner: Optional[Account] = field(metadata=AirModel.FIELD_FOREIGN_KEY, repr=False)
    platform_information: DataDict = field(repr=False)
    simulation_engine_versions: List[str] = field(repr=False)
    simulator_image: Image = field(metadata=AirModel.FIELD_FOREIGN_KEY)
    simulator_resources: DataDict = field(repr=False)
    emulation_params: DataDict = field(repr=False)

    @classmethod
    def get_model_api(cls):
        """
        Returns the respective `AirModelAPI` type for this model.
        """
        return ManifestEndpointApi


class ManifestEndpointApi(
    mixins.ListApiMixin[Manifest],
    mixins.CreateApiMixin[Manifest],
    mixins.GetApiMixin[Manifest],
    mixins.PatchApiMixin[Manifest],
    mixins.PutApiMixin[Manifest],
    mixins.DeleteApiMixin,
    BaseEndpointApi[Manifest],
):
    API_PATH = 'manifests'
    model = Manifest

    def create(
        self,
        artifacts_directory: str,
        configure_node_properties: DataDict,
        configure_simulator: DataDict,
        docker_run_parameters: DataDict,
        emulation_type: str,
        organization: Union[Organization, PrimaryKey],
        simulator_image: Union[Image, PrimaryKey],
        simulator_resources: DataDict,
        artifacts_directory_max_size_gb: Optional[int] = None,
        boot_group: Optional[int] = None,
        platform_information: Optional[DataDict] = None,
        simulation_engine_versions: Optional[List[str]] = None,
        emulation_params: Optional[DataDict] = None,
    ) -> Manifest:
        payload: DataDict = {
            'artifacts_directory': artifacts_directory,
            'configure_node_properties': configure_node_properties,
            'configure_simulator': configure_simulator,
            'docker_run_parameters': docker_run_parameters,
            'emulation_type': emulation_type,
            'organization': organization,
            'simulator_image': simulator_image,
            'simulator_resources': simulator_resources,
        }
        for key, value in (
            ('artifacts_directory_max_size_gb', artifacts_directory_max_size_gb),
            ('boot_group', boot_group),
            ('platform_information', platform_information),
            ('simulation_engine_versions', simulation_engine_versions),
            ('emulation_params', emulation_params),
        ):
            if value is not None:  # Don't include optional None fields in payload.
                payload[key] = value
        return super().create(**payload)
